<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:s="http://www.schema-library.org/show/elements/type" 
                xmlns:s2="http://www.schema-library.org/show/substitutionGroup/elements"
                xmlns:rw="http://www.schema-library.org/rewrite"
                xmlns:red="http://www.schema-library.org/reduce" 
                xmlns:sic="http://www.schema-library.org/schema-in-context" 
                xmlns:h="http://www.schema-library.org/input-schema-and-its-dependency-hierarchy"
                xmlns:removed="http://www.remove.org" 
                xmlns:xs="http://www.w3.org/2001/XMLSchema" 
                exclude-result-prefixes="#all" 
                version="2.0">

    <!-- 
        A complexType may extend another. This program merges the
        stuff in the base type into the child type.  Thus, the child
        type is standalone (no longer dependent on a base type).
        
        Roger L. Costello
    -->
    
    <xsl:key name="ids" match="*[@_id]" use="@_id"/>

    <xsl:function name="s:inline-complexContent-derive-by-extension" as="document-node()">
        <xsl:param name="schemas" as="document-node()"/>

        <xsl:variable name="updated-schemas">
            <schemas>
                <xsl:apply-templates select="$schemas/schemas/xs:schema" mode="inline-derive-by-extension"/>
            </schemas>
        </xsl:variable>

        <xsl:sequence select="$updated-schemas"/>

    </xsl:function>

    <xsl:template match="xs:schema" mode="inline-derive-by-extension">

        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-derive-by-extension" />
        </xsl:copy>

    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:complexContent/xs:extension]" mode="inline-derive-by-extension">
        
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <!-- The attribute, mixed, can be on complexContent so I
                 move it up onto the complexType element. -->
            <xsl:copy-of select="xs:complexContent/@mixed"/>
            <xs:sequence>
                <xsl:for-each select="xs:complexContent/xs:extension">
                    <xsl:variable name="type" select="key('ids', @idref)"/>
                    <xsl:apply-templates select="$type" mode="get-element-content-of-complexType" />
                </xsl:for-each>
                <xsl:choose>
                    <xsl:when test="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)">
                        <xsl:apply-templates select="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)" mode="inline-derive-by-extension" />
                    </xsl:when>
                </xsl:choose>
            </xs:sequence>
            <xsl:for-each select="xs:complexContent/xs:extension">
                <xsl:variable name="type" select="key('ids', @idref)"/>
                <xsl:apply-templates select="$type" mode="get-attribute-content-of-complexType" />
            </xsl:for-each>
            <xsl:choose>
                <xsl:when test="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)">
                    <xsl:sequence select="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)/following-sibling::xs:*" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:sequence select="xs:complexContent/xs:extension/xs:*" />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>
        
    </xsl:template>
    
    <xsl:template match="node()" mode="inline-derive-by-extension">
        
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="inline-derive-by-extension" />
        </xsl:copy>
        
    </xsl:template>
    
    <!-- ****************************************************************************** -->
    
    <xsl:template match="xs:complexType[xs:sequence]" mode="get-element-content-of-complexType">
        
        <xsl:sequence select="xs:sequence" />
        
    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:choice]" mode="get-element-content-of-complexType">
        
        <xsl:sequence select="xs:choice" />
        
    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:all]" mode="get-element-content-of-complexType">
        
        <xsl:sequence select="xs:all" />
        
    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:complexContent/xs:extension]" mode="get-element-content-of-complexType">
        
        <xsl:choose>
            <xsl:when test="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)">
                <xs:sequence>
                    <xsl:for-each select="xs:complexContent/xs:extension">
                        <xsl:variable name="type" select="key('ids', @idref)"/>
                        <xsl:apply-templates select="$type" mode="get-element-content-of-complexType" />
                    </xsl:for-each>
                    <xsl:apply-templates select="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)" mode="inline-derive-by-extension" />
                </xs:sequence>
            </xsl:when>
            <xsl:otherwise>
                <xsl:for-each select="xs:complexContent/xs:extension">
                    <xsl:variable name="type" select="key('ids', @idref)"/>
                    <xsl:apply-templates select="$type" mode="get-element-content-of-complexType" />
                </xsl:for-each>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    
    <!-- ****************************************************************************** -->
    
    <xsl:template match="xs:complexType[xs:sequence]" mode="get-attribute-content-of-complexType">
        
        <xsl:sequence select="xs:sequence/following-sibling::xs:*" />
        
    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:choice]" mode="get-attribute-content-of-complexType">
        
        <xsl:sequence select="xs:choice/following-sibling::xs:*" />
        
    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:all]" mode="get-attribute-content-of-complexType">
        
        <xsl:sequence select="xs:all/following-sibling::xs:*" />
        
    </xsl:template>
    
    <xsl:template match="xs:complexType[xs:complexContent/xs:extension]" mode="get-attribute-content-of-complexType">
        
        <xsl:for-each select="xs:complexContent/xs:extension">
            <xsl:variable name="type" select="key('ids', @idref)"/>
            <xsl:apply-templates select="$type" mode="get-attribute-content-of-complexType" />
        </xsl:for-each>
        <xsl:choose>
            <xsl:when test="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)">
                <xsl:sequence select="xs:complexContent/xs:extension/(xs:sequence | xs:choice | xs:all)/following-sibling::xs:*" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="xs:complexContent/xs:extension/xs:*" />
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>

</xsl:stylesheet>
