<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:h="http://www.schema-library.org/input-schema-and-its-dependency-hierarchy"
                exclude-result-prefixes="#all"
                version="2.0">
    
    <!-- 
        
        Tool that inputs an XML Schema and recursively inputs the included and imported XML Schemas
        Author: Roger L. Costello
        Version 1.0
        March 20, 2012
        
    -->
    
    <xsl:function name="h:input-schemas" as="document-node()">
        <xsl:param name="schema" as="document-node()" />
        
        <xsl:variable name="schemas">
            <xsl:apply-templates select="$schema/xs:schema" mode="input-schemas"/>
        </xsl:variable>
        
        <xsl:sequence select="$schemas" />
        
    </xsl:function>
    
    <xsl:template match="xs:schema" mode="input-schemas"> 
        
        <xsl:variable name="schema" select="h:add-id(.)" />
        
        <schemas> 
            <xsl:apply-templates select="$process-graph">
                <xsl:with-param name="items-to-process" select="$schema" />
                <xsl:with-param name="items-already-processed-or-in-queue-for-processing" select="$schema" />
                <xsl:with-param name="function-to-process-current-item" select="$h:function-to-process-current-item" />
                <xsl:with-param name="function-to-determine-new-items-to-process" select="$h:function-to-determine-new-items-to-process" />
            </xsl:apply-templates>         
        </schemas>  
    </xsl:template>
    
    
    <xsl:variable name="h:function-to-determine-new-items-to-process" select="document('')//h:function-to-determine-new-items-to-process[1]" />
    <xsl:variable name="h:function-to-process-current-item" select="document('')//h:function-to-process-current-item[1]" />
    
    <h:function-to-determine-new-items-to-process />
    <h:function-to-process-current-item />
    
    
    <!-- 
        The new schemas are those pointed at by xs:include, xs:import, and xs:redefine which are not already on
        the list for processing.
    -->
    
    <xsl:template match="h:function-to-determine-new-items-to-process">
        <xsl:param name="item" />
        <xsl:param name="items-already-processed-or-in-queue-for-processing" />
                
        <xsl:variable name="items" select="$item/(xs:include[@schemaLocation] | xs:import[@schemaLocation] | xs:redefine[@schemaLocation])" />
        <xsl:sequence select="h:helper-function-to-determine-new-items-to-process($items, $items-already-processed-or-in-queue-for-processing)" />
        
    </xsl:template>
    
    <xsl:function name="h:helper-function-to-determine-new-items-to-process">
        <xsl:param name="items" />
        <xsl:param name="items-already-processed-or-in-queue-for-processing" />
        
        <xsl:choose>
            <xsl:when test="empty($items)" />
            <xsl:otherwise>
                <xsl:for-each select="$items[1]">
                    <xsl:variable name="url" select="resolve-uri(@schemaLocation, base-uri(.))" />
                    <xsl:choose>
                        <xsl:when test="doc-available($url)">
                            <xsl:variable name="referenced-schema" select="h:add-id(document(@schemaLocation, .)/xs:schema)" />
                            
                            <xsl:choose>
                                <xsl:when test="not($referenced-schema/@_id = $items-already-processed-or-in-queue-for-processing/@_id)">
                                    <xsl:sequence select="$referenced-schema" />
                                    <xsl:sequence select="h:helper-function-to-determine-new-items-to-process($items[position() gt 1], ($items-already-processed-or-in-queue-for-processing, $referenced-schema))" />
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:sequence select="h:helper-function-to-determine-new-items-to-process($items[position() gt 1], $items-already-processed-or-in-queue-for-processing)" />
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="error(QName('http://www.w3.org/2001/XMLSchema', local-name(.)), concat('This XML Schema: ', base-uri(.), ' could not retrieve this XML Schema: ', @schemaLocation))" />
                        </xsl:otherwise>
                    </xsl:choose>
                    
 
                </xsl:for-each>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
    
    
    <!-- 
       Output the XML Schema
    -->
    <xsl:template match="h:function-to-process-current-item">
        <xsl:param name="item" />
        
        <xsl:sequence select="$item" />
        
    </xsl:template> 
    
    
    <!-- 
        Add to the schema @xml:base and an identifier 
    -->
    <xsl:function name="h:add-id" as="element(xs:schema)">
        <xsl:param name="schema" as="element(xs:schema)" />
        
        <xsl:for-each select="$schema">
            <xsl:variable name="_id" select="generate-id(.)" />
            <xsl:copy>
                <xsl:copy-of select="@* except (@_id, @xml:base)" />
                <xsl:attribute name="_id"><xsl:value-of select="$_id" /></xsl:attribute>
                <xsl:attribute name="xml:base"><xsl:value-of select="base-uri(.)" /></xsl:attribute>
                <xsl:sequence select="node() | comment() | processing-instruction()" />
            </xsl:copy>
        </xsl:for-each>
        
    </xsl:function>
    
    
</xsl:stylesheet>